@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2009 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -17,48 +17,30 @@
 */
 
 #include "Dialog.h"
-
-#include <Q3CString>
-#include <Q3ValueList>
+#include <KAction>
+#include <KActionCollection>
+#include <QList>
 #include <QCloseEvent>
 #include <QDir>
 #include <QDockWidget>
-#include <QEvent>
 #include <QHBoxLayout>
 #include <QMainWindow>
-#include <QMoveEvent>
-#include <QPixmap>
-#include <QResizeEvent>
 #include <QTimeEdit>
-#include <QTimer>
 #include <QVBoxLayout>
 #include <kacceleratormanager.h>
-#include <kconfig.h>
-#include <kdebug.h>
-#include <kglobal.h>
 #include <kguiitem.h>
-#include <kiconloader.h>
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kpushbutton.h>
-#include <kstandarddirs.h>
 #include <ktextedit.h>
-#include <q3accel.h>
-#include <q3popupmenu.h>
+#include <QMenu>
 #include <qapplication.h>
 #include <qcursor.h>
-#include <qdialog.h>
-#include <qeventloop.h>
 #include <qfile.h>
 #include <qfileinfo.h>
-#include <qglobal.h>
 #include <qlabel.h>
-#include <qlayout.h>
-#include <qobject.h>
 #include <qpoint.h>
-#include <qpushbutton.h>
-#include <qtooltip.h>
 
 #ifdef HAVE_NEPOMUK
 #   include <nepomuk/kratingwidget.h>
@@ -69,7 +51,7 @@
 #include "DB/ImageInfo.h"
 #include "DB/Result.h"
 #include "DB/ResultId.h"
-#include "ImagePreview.h"
+#include "ImagePreviewWidget.h"
 #include "KDateEdit.h"
 #include "ListSelect.h"
 #include "MainWindow/DeleteDialog.h"
@@ -84,8 +66,13 @@
 
 using Utilities::StringSet;
 
+/**
+ * \class AnnotationDialog::Dialog
+ * \brief QDialog subclass used for tagging images
+ */
+
 AnnotationDialog::Dialog::Dialog( QWidget* parent )
-    : QDialog( parent ), _viewer(0), _ratingChanged(false)
+    : QDialog( parent ), _ratingChanged(false)
 {
     Utilities::ShowBusyCursor dummy;
     ShortCutManager shortCutManager;
@@ -96,7 +83,7 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent )
     layout->addWidget( _dockWindow );
 
     // -------------------------------------------------- Dock widgets
-    QDockWidget* dock = createDock( i18n("Label and Dates"), QString::fromLatin1("Label and Datas"), Qt::TopDockWidgetArea, createDateWidget(shortCutManager) );
+    QDockWidget* dock = createDock( i18n("Label and Dates"), QString::fromLatin1("Label and Dates"), Qt::TopDockWidgetArea, createDateWidget(shortCutManager) );
 
     createDock( i18n("Image Preview"), QString::fromLatin1("Image Preview"), Qt::TopDockWidgetArea, createPreviewWidget() );
 
@@ -107,8 +94,8 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent )
     shortCutManager.addDock( dock, _description );
 
     // -------------------------------------------------- Categrories
-    Q3ValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
-    for( Q3ValueList<DB::CategoryPtr>::ConstIterator categoryIt = categories.begin(); categoryIt != categories.end(); ++categoryIt ) {
+     QList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
+     for( QList<DB::CategoryPtr>::ConstIterator categoryIt = categories.constBegin(); categoryIt != categories.constEnd(); ++categoryIt ) {
         ListSelect* sel = createListSel( *categoryIt );
         QDockWidget* dock = createDock( (*categoryIt)->text(), (*categoryIt)->name(), Qt::BottomDockWidgetArea, sel );
         shortCutManager.addDock( dock, sel->lineEdit() );
@@ -120,45 +107,55 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent )
     QHBoxLayout* lay1 = new QHBoxLayout;
     layout->addLayout( lay1 );
 
-    _revertBut = new QPushButton( i18n("Revert This Item"), this );
+    _revertBut = new KPushButton( i18n("Revert This Item"), this );
     KAcceleratorManager::setNoAccel(_revertBut);
     lay1->addWidget( _revertBut );
 
-    QPushButton* clearBut = new KPushButton( KGuiItem(i18n("Clear Form"),QApplication::isRightToLeft()
+    _clearBut = new KPushButton( KGuiItem(i18n("Clear Form"),QApplication::isRightToLeft()
                                              ? QString::fromLatin1("clear_left")
                                              : QString::fromLatin1("locationbar_erase")), this );
-    KAcceleratorManager::setNoAccel(clearBut);
-    lay1->addWidget( clearBut );
+    KAcceleratorManager::setNoAccel(_clearBut);
+    lay1->addWidget( _clearBut );
 
-    QPushButton* optionsBut = new QPushButton( i18n("Options" ), this );
+    KPushButton* optionsBut = new KPushButton( i18n("Options..." ), this );
     KAcceleratorManager::setNoAccel(optionsBut);
     lay1->addWidget( optionsBut );
 
     lay1->addStretch(1);
 
-    _okBut = new KPushButton( KStandardGuiItem::ok(), this );
+    _okBut = new KPushButton( i18n("&Done"), this );
     lay1->addWidget( _okBut );
 
-    QPushButton* cancelBut = new KPushButton( KStandardGuiItem::cancel(), this );
+    _continueLaterBut = new KPushButton( i18n("Continue &Later"), this );
+    lay1->addWidget( _continueLaterBut );
+
+    KPushButton* cancelBut = new KPushButton( KStandardGuiItem::cancel(), this );
     lay1->addWidget( cancelBut );
 
     // It is unfortunately not possible to ask KAcceleratorManager not to setup the OK and cancel keys.
     shortCutManager.addTaken( i18n("&Search") );
     shortCutManager.addTaken( _okBut->text() );
+    shortCutManager.addTaken( _continueLaterBut->text());
     shortCutManager.addTaken( cancelBut->text() );
 
     connect( _revertBut, SIGNAL( clicked() ), this, SLOT( slotRevert() ) );
-    connect( _okBut, SIGNAL( clicked() ), this, SLOT( slotOK() ) );
+    connect( _okBut, SIGNAL( clicked() ), this, SLOT( doneTagging() ) );
+    connect( _continueLaterBut, SIGNAL( clicked() ), this, SLOT( continueLater() ) );
     connect( cancelBut, SIGNAL( clicked() ), this, SLOT( reject() ) );
-    connect( clearBut, SIGNAL( clicked() ), this, SLOT(slotClear() ) );
+    connect( _clearBut, SIGNAL( clicked() ), this, SLOT(slotClear() ) );
     connect( optionsBut, SIGNAL( clicked() ), this, SLOT( slotOptions() ) );
+    
+    connect( _preview, SIGNAL( imageRotated( int ) ), this, SLOT( rotate( int ) ) );
+    connect( _preview, SIGNAL( indexChanged( int ) ), this, SLOT( slotIndexChanged( int ) ) );
+    connect( _preview, SIGNAL( imageDeleted( const DB::ImageInfo& ) ), this, SLOT( slotDeleteImage() ) );
+    connect( _preview, SIGNAL( copyPrevClicked() ), this, SLOT( slotCopyPrevious() ) );
 
     // Disable so no button accept return (which would break with the line edits)
     _revertBut->setAutoDefault( false );
     _okBut->setAutoDefault( false );
-    _delBut->setAutoDefault( false );
+    _continueLaterBut->setAutoDefault( false );
     cancelBut->setAutoDefault( false );
-    clearBut->setAutoDefault( false );
+    _clearBut->setAutoDefault( false );
     optionsBut->setAutoDefault( false );
 
     _optionList.setAutoDelete( true );
@@ -166,6 +163,8 @@ AnnotationDialog::Dialog::Dialog( QWidget* parent )
     _dockWindowCleanState = _dockWindow->saveState();
 
     loadWindowLayout();
+    
+    _current = -1;
 
     setGeometry( Settings::SettingsData::instance()->windowGeometry( Settings::AnnotationDialog ) );
 
@@ -238,7 +237,7 @@ QWidget* AnnotationDialog::Dialog::createDateWidget(ShortCutManager& shortCutMan
     lay7->addStretch(1);
     _time->hide();
 
-    _addTime= new QPushButton(i18n("Add Time Info..."));
+    _addTime= new KPushButton(i18n("Add Time Info..."));
     _addTime->setProperty( "WantsFocus", true );
     lay7->addWidget( _addTime );
     lay7->addStretch(1);
@@ -247,6 +246,7 @@ QWidget* AnnotationDialog::Dialog::createDateWidget(ShortCutManager& shortCutMan
 
 #ifdef HAVE_NEPOMUK
     _rating = new KRatingWidget( this );
+    _rating->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
     lay2->addStretch(1);
     lay2->addWidget( _rating, 0, Qt::AlignCenter );
     connect( _rating, SIGNAL( ratingChanged( unsigned int ) ), this, SLOT( slotRatingChanged( unsigned int ) ) );
@@ -257,159 +257,40 @@ QWidget* AnnotationDialog::Dialog::createDateWidget(ShortCutManager& shortCutMan
     return top;
 }
 
-
 QWidget* AnnotationDialog::Dialog::createPreviewWidget()
 {
-    QWidget* top = new QWidget;
-    QVBoxLayout* layout = new QVBoxLayout( top );
-
-    _preview = new ImagePreview( top );
-    layout->addWidget( _preview, 1 );
-
-    QHBoxLayout* hlay = new QHBoxLayout;
-    layout->addLayout( hlay );
-    hlay->addStretch(1);
-
-    _prevBut = new QPushButton( top );
-    _prevBut->setIcon( KIcon( QString::fromLatin1( "arrow-left" ) ) );
-    _prevBut->setFixedWidth( 40 );
-    hlay->addWidget( _prevBut );
-    _prevBut->setToolTip( i18n("Annotate previous image") );
-
-    _nextBut = new QPushButton( top );
-    _nextBut->setIcon( KIcon( QString::fromLatin1( "arrow-right" ) ) );
-    _nextBut->setFixedWidth( 40 );
-    hlay->addWidget( _nextBut );
-    _nextBut->setToolTip( i18n("Annotate next image") );
-
-    hlay->addStretch(1);
-
-    _rotateLeft = new QPushButton( top );
-    hlay->addWidget( _rotateLeft );
-    _rotateLeft->setIcon( KIcon( QString::fromLatin1( "object-rotate-left" ) ) );
-    _rotateLeft->setFixedWidth( 40 );
-    _rotateLeft->setToolTip( i18n("Rotate counterclockwise") );
-
-    _rotateRight = new QPushButton( top );
-    hlay->addWidget( _rotateRight );
-    _rotateRight->setIcon( KIcon( QString::fromLatin1( "object-rotate-right" ) ) );
-    _rotateRight->setFixedWidth( 40 );
-    _rotateRight->setToolTip( i18n("Rotate clockwise") );
-
-    _copyPreviousBut = new QPushButton( top );
-    hlay->addWidget( _copyPreviousBut );
-    _copyPreviousBut->setIcon( KIcon( QString::fromLatin1( "go-bottom" ) ) );
-    _copyPreviousBut->setFixedWidth( 40 );
-    connect( _copyPreviousBut, SIGNAL( clicked() ), this, SLOT( slotCopyPrevious() ) );
-    _copyPreviousBut->setToolTip( i18n("Copy tags from previously tagged image") );
-
-    hlay->addStretch( 1 );
-    _delBut = new QPushButton( top );
-    _delBut->setIcon( KIcon( QString::fromLatin1( "edit-delete" ) ) );
-    hlay->addWidget( _delBut );
-    connect( _delBut, SIGNAL( clicked() ), this, SLOT( slotDeleteImage() ) );
-    _delBut->setToolTip( i18n("Delete image") );
-
-    hlay->addStretch(1);
-
-    return top;
+    _preview = new ImagePreviewWidget();
+    return _preview;
 }
 
-
 void AnnotationDialog::Dialog::slotRevert()
 {
     if ( _setup == InputSingleImageConfigMode )
         load();
 }
 
-void AnnotationDialog::Dialog::slotPrev()
+void AnnotationDialog::Dialog::slotIndexChanged( int index )
 {
-    if ( _setup != InputSingleImageConfigMode )
+  if ( _setup != InputSingleImageConfigMode )
         return;
-
-    writeToInfo();
-    if ( _current == 0 )
-        return;
-
-    _current--;
-    if ( _setup == InputSingleImageConfigMode && _current != 0 )
-        _preview->anticipate(_editList[ _current-1 ]);
+    
+    if(_current >= 0 )
+      writeToInfo();
+    
+    _current = index;
+    
     load();
 }
 
-void AnnotationDialog::Dialog::slotNext()
+void AnnotationDialog::Dialog::doneTagging()
 {
-    if ( _setup != InputSingleImageConfigMode )
-        return;
-
-    if ( _current != -1 ) {
-        writeToInfo();
-    }
-    if ( _current == (int)_origList.count()-1 )
-        return;
-
-    _current++;
-    if ( _setup == InputSingleImageConfigMode && _current != (int)_origList.count()-1 )
-        _preview->anticipate(_editList[ _current+1 ]);
-    load();
-}
-
-void AnnotationDialog::Dialog::slotOK()
-{
-    if (_origList.isEmpty()) {
-        // all images are deleted.
-        QDialog::accept();
-        return;
-    }
-
-// I need to check for the changes first, as the case for _setup == InputSingleImageConfigMode, saves to the _origList,
-    // and we can thus not check for changes anymore
-    const bool anyChanges = hasChanges();
-
-    if ( _setup == InputSingleImageConfigMode )  {
-        writeToInfo();
-        for ( int i = 0; i < _editList.count(); ++i )  {
-            *(_origList[i]) = _editList[i];
+    saveAndClose();
+    if ( Settings::SettingsData::instance()->hasUntaggedCategoryFeatureConfigured() ) {
+        for( DB::ImageInfoListIterator it = _origList.begin(); it != _origList.end(); ++it ) {
+            (*it)->removeCategoryInfo( Settings::SettingsData::instance()->untaggedCategory(),
+                                      Settings::SettingsData::instance()->untaggedTag() );
         }
     }
-    else if ( _setup == InputMultiImageConfigMode ) {
-        for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
-            (*it)->slotReturn();
-        }
-
-        for( DB::ImageInfoListConstIterator it = _origList.constBegin(); it != _origList.constEnd(); ++it ) {
-            DB::ImageInfoPtr info = *it;
-            info->delaySavingChanges(true);
-            info->rotate( _preview->angle() );
-            if ( !_startDate->date().isNull() )
-                info->setDate( DB::ImageDate( _startDate->date(), _endDate->date(), _time->time() ) );
-
-            for( Q3PtrListIterator<ListSelect> listSelectIt( _optionList ); *listSelectIt; ++listSelectIt ) {
-                info->addCategoryInfo( (*listSelectIt)->category(), (*listSelectIt)->itemsOn() );
-                info->removeCategoryInfo( (*listSelectIt)->category(), (*listSelectIt)->itemsOff() );
-            }
-
-            if ( !_imageLabel->text().isEmpty() ) {
-                info->setLabel( _imageLabel->text() );
-            }
-
-
-            if ( !_description->toPlainText().isEmpty() ) {
-                info->setDescription( _description->toPlainText() );
-            }
-
-            info->delaySavingChanges(false);
-        }
-    }
-    _accept = QDialog::Accepted;
-
-    // I shouldn't emit changed before I've actually commited the changes, otherwise the listeners will act on the old data.
-    if ( anyChanges ) {
-        MainWindow::DirtyIndicator::markDirty();
-        _thumbnailTextShouldReload = true;
-    }
-
-    QDialog::accept();
 }
 
 /*
@@ -454,23 +335,16 @@ void AnnotationDialog::Dialog::load()
     _description->setText( info.description() );
 
 #ifdef HAVE_NEPOMUK
-    _rating->setRating( qMax( static_cast<short int>(0), info.rating() ) );
+    if ( _setup == InputSingleImageConfigMode )
+        _rating->setRating( qMax( static_cast<short int>(0), info.rating() ) );
     _ratingChanged = false;
 #endif
 
     for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
         (*it)->setSelection( info.itemsOfCategory( (*it)->category() ) );
+        (*it)->rePopulate();
     }
 
-    _nextBut->setEnabled( _current != (int)_origList.count()-1 );
-    _prevBut->setEnabled( _current != 0 );
-    _copyPreviousBut->setEnabled( _current != 0 );
-
-    _preview->setImage( info );
-
-    if ( _viewer )
-        _viewer->load( Utilities::infoListToStringList(_origList), _current );
-
     if ( _setup == InputSingleImageConfigMode )
         setWindowTitle( i18n("KPhotoAlbum Annotations (%1/%2)", _current+1, _origList.count() ) );
 }
@@ -511,6 +385,12 @@ void AnnotationDialog::Dialog::writeToInfo()
 
 int AnnotationDialog::Dialog::configure( DB::ImageInfoList list, bool oneAtATime )
 {
+    if ( Settings::SettingsData::instance()->hasUntaggedCategoryFeatureConfigured() ) {
+        DB::ImageDB::instance()->categoryCollection()->categoryForName( Settings::SettingsData::instance()->untaggedCategory() )
+            ->addItem(Settings::SettingsData::instance()->untaggedTag() );
+    }
+    
+
     if ( oneAtATime )
         _setup = InputSingleImageConfigMode;
     else
@@ -526,20 +406,19 @@ int AnnotationDialog::Dialog::configure( DB::ImageInfoList list, bool oneAtATime
     setup();
 
     if ( oneAtATime )  {
-        _current = -1;
-        slotNext();
-#ifdef HAVE_NEPOMUK
-        _rating->setEnabled( true );
-#endif
+        _current = 0;
+        _preview->configure( &_editList, true );
+        load();
     }
     else {
+        _preview->configure( &_editList, false );
         _startDate->setDate( QDate() );
         _endDate->setDate( QDate() );
         _time->hide();
         _addTime->show();
 #ifdef HAVE_NEPOMUK
-        _rating->setEnabled( false );
         _rating->setRating( 0 );
+        _ratingChanged = false;
 #endif
 
         for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it )
@@ -548,15 +427,13 @@ int AnnotationDialog::Dialog::configure( DB::ImageInfoList list, bool oneAtATime
         _imageLabel->setText( QString::fromLatin1("") );
         _description->setText( QString::fromLatin1("") );
 
-        _prevBut->setEnabled( false );
-        _nextBut->setEnabled( false );
-        _copyPreviousBut->setEnabled( false );
     }
 
     _thumbnailShouldReload = false;
     _thumbnailTextShouldReload = false;
 
     showHelpDialog( oneAtATime ? InputSingleImageConfigMode : InputMultiImageConfigMode );
+
     return exec();
 }
 
@@ -567,6 +444,14 @@ DB::ImageSearchInfo AnnotationDialog::Dialog::search( DB::ImageSearchInfo* searc
         _oldSearch = *search;
 
     setup();
+
+    _preview->setImage(Utilities::locateDataFile(QString::fromLatin1("pics/search.jpg")));
+
+#ifdef HAVE_NEPOMUK
+        _rating->setRating( 0 );
+        _ratingChanged = false ;
+#endif
+
     showHelpDialog( SearchMode );
     int ok = exec();
     if ( ok == QDialog::Accepted )  {
@@ -576,9 +461,16 @@ DB::ImageSearchInfo AnnotationDialog::Dialog::search( DB::ImageSearchInfo* searc
                                       _imageLabel->text(), _description->toPlainText() );
 
         for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
-            _oldSearch.setOption( (*it)->category(), (*it)->text() );
+            _oldSearch.setCategoryMatchText( (*it)->category(), (*it)->text() );
         }
-
+#ifdef HAVE_NEPOMUK
+        //FIXME: for the user to search for 0-rated images, he must first change the rating to anything > 0
+        //then change back to 0 .
+        if( _ratingChanged)
+          _oldSearch.setRating( _rating->rating() );
+        
+        _ratingChanged = false;
+#endif
         return _oldSearch;
     }
     else
@@ -595,30 +487,21 @@ void AnnotationDialog::Dialog::setup()
 
     if ( _setup == SearchMode )  {
         _okBut->setGuiItem( KGuiItem(i18n("&Search"), QString::fromLatin1("find")) );
+        _continueLaterBut->hide();
         _revertBut->hide();
+        _clearBut->show();
         setWindowTitle( i18n("Search") );
         loadInfo( _oldSearch );
-        _preview->setImage(Utilities::locateDataFile(QString::fromLatin1("pics/search.jpg")));
-        _nextBut->setEnabled( false );
-        _prevBut->setEnabled( false );
-        _rotateLeft->setEnabled( false );
-        _rotateRight->setEnabled( false );
     }
     else {
-        _okBut->setGuiItem( KStandardGuiItem::ok() );
+        _okBut->setText( i18n("Done") );
+        _continueLaterBut->show();
         _revertBut->setEnabled( _setup == InputSingleImageConfigMode );
+        _clearBut->hide();
         _revertBut->show();
         setWindowTitle( i18n("Annotations") );
-        if ( _setup == InputMultiImageConfigMode ) {
-            _preview->setImage(Utilities::locateDataFile(QString::fromLatin1("pics/multiconfig.jpg")));
-        }
-        _rotateLeft->setEnabled( true );
-        _rotateRight->setEnabled( true );
     }
 
-    _delBut->setEnabled( _setup == InputSingleImageConfigMode );
-    _copyPreviousBut->setEnabled( _setup == InputSingleImageConfigMode );
-
     for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it )
         (*it)->setMode( _setup );
 }
@@ -635,18 +518,13 @@ void AnnotationDialog::Dialog::loadInfo( const DB::ImageSearchInfo& info )
     _endDate->setDate( info.date().end().date() );
 
     for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
-        (*it)->setText( info.option( (*it)->category() ) );
+        (*it)->setText( info.categoryMatchText( (*it)->category() ) );
     }
 
     _imageLabel->setText( info.label() );
     _description->setText( info.description() );
 }
 
-void AnnotationDialog::Dialog::viewerDestroyed()
-{
-    _viewer = 0;
-}
-
 void AnnotationDialog::Dialog::slotOptions()
 {
     QMenu menu( this );
@@ -716,14 +594,14 @@ AnnotationDialog::ListSelect* AnnotationDialog::Dialog::createListSel( const DB:
 
 void AnnotationDialog::Dialog::slotDeleteOption( DB::Category* category, const QString& value )
 {
-    for( Q3ValueListIterator<DB::ImageInfo> it = _editList.begin(); it != _editList.end(); ++it ) {
+    for( QList<DB::ImageInfo>::Iterator it = _editList.begin(); it != _editList.end(); ++it ) {
         (*it).removeCategoryInfo( category->name(), value );
     }
 }
 
 void AnnotationDialog::Dialog::slotRenameOption( DB::Category* category, const QString& oldValue, const QString& newValue )
 {
-    for( Q3ValueListIterator<DB::ImageInfo> it = _editList.begin(); it != _editList.end(); ++it ) {
+    for( QList<DB::ImageInfo>::Iterator it = _editList.begin(); it != _editList.end(); ++it ) {
         (*it).renameItem( category->name(), oldValue, newValue );
     }
 }
@@ -767,31 +645,21 @@ bool AnnotationDialog::Dialog::hasChanges()
 
         changed |= ( !_imageLabel->text().isEmpty() );
         changed |= ( !_description->toPlainText().isEmpty() );
+        changed |= _ratingChanged;
     }
     return changed;
 }
 
-void AnnotationDialog::Dialog::rotateLeft()
-{
-    rotate(-90);
-}
-
-void AnnotationDialog::Dialog::rotateRight()
-{
-    rotate(90);
-}
-
 void AnnotationDialog::Dialog::rotate( int angle )
 {
     _thumbnailShouldReload = true;
     if ( _setup == InputMultiImageConfigMode ) {
-        // In slotOK the preview will be queried for its angle.
+        // In doneTagging the preview will be queried for its angle.
     }
     else {
         DB::ImageInfo& info = _editList[ _current ];
         info.rotate(angle);
     }
-    _preview->rotate( angle );
 }
 
 bool AnnotationDialog::Dialog::thumbnailShouldReload() const
@@ -813,7 +681,10 @@ void AnnotationDialog::Dialog::slotAddTimeInfo()
 void AnnotationDialog::Dialog::slotDeleteImage()
 {
     Q_ASSERT( _setup != SearchMode );
-
+    
+    if( _setup == InputMultiImageConfigMode )  //TODO: probably delete here should mean remove from selection
+      return;
+    
     MainWindow::DeleteDialog dialog( this );
     DB::ImageInfoPtr info = _origList[_current];
 
@@ -825,11 +696,11 @@ void AnnotationDialog::Dialog::slotDeleteImage()
         return;
 
     _origList.remove( info );
-    _editList.remove( _editList.at( _current ) );
+    _editList.removeAll( _editList.at( _current ) );
     _thumbnailShouldReload = true;
     MainWindow::DirtyIndicator::markDirty();
     if ( _origList.count() == 0 ) {
-        slotOK();
+        doneTagging();
         return;
     }
     if ( _current == (int)_origList.count() ) // we deleted the last image
@@ -919,7 +790,7 @@ void AnnotationDialog::Dialog::setupFocus()
 
     // Finally set focus on the first list select
     for( QList<QWidget*>::Iterator orderedIt = orderedList.begin(); orderedIt != orderedList.end(); ++orderedIt ) {
-        if ( (*orderedIt)->property("FocusCandidate").isValid() ) {
+        if ( (*orderedIt)->property("FocusCandidate").isValid() && (*orderedIt)->isVisible() ) {
             (*orderedIt)->setFocus();
             break;
         }
@@ -974,15 +845,15 @@ void AnnotationDialog::Dialog::setupActions()
     action->setShortcut( Qt::CTRL+Qt::Key_S );
 
 
-    action = _actions->addAction( QString::fromLatin1("annotationdialog-next-image"),  this, SLOT( slotNext() ) );
+    action = _actions->addAction( QString::fromLatin1("annotationdialog-next-image"),  _preview, SLOT( slotNext() ) );
     action->setText(  i18n("Annotate Next") );
     action->setShortcut(  Qt::Key_PageDown );
 
-    action = _actions->addAction( QString::fromLatin1("annotationdialog-prev-image"),  this, SLOT( slotPrev() ) );
+    action = _actions->addAction( QString::fromLatin1("annotationdialog-prev-image"),  _preview, SLOT( slotPrev() ) );
     action->setText(  i18n("Annotate Previous") );
     action->setShortcut(  Qt::Key_PageUp );
 
-    action = _actions->addAction( QString::fromLatin1("annotationdialog-OK-dialog"),  this, SLOT( slotOK() ) );
+    action = _actions->addAction( QString::fromLatin1("annotationdialog-OK-dialog"),  this, SLOT( doneTagging() ) );
     action->setText(  i18n("OK dialog") );
     action->setShortcut(  Qt::CTRL+Qt::Key_Return );
 
@@ -994,17 +865,12 @@ void AnnotationDialog::Dialog::setupActions()
     action->setText(  i18n("Copy tags from previous image") );
     action->setShortcut(  Qt::ALT+Qt::Key_Insert );
 
-    action = _actions->addAction( QString::fromLatin1("annotationdialog-rotate-left"),  this, SLOT( rotateLeft() ) );
+    action = _actions->addAction( QString::fromLatin1("annotationdialog-rotate-left"),  _preview, SLOT( rotateLeft() ) );
     action->setText(  i18n("Rotate counterclockwise") );
 
-    action = _actions->addAction( QString::fromLatin1("annotationdialog-rotate-right"),  this, SLOT( rotateRight() ) );
+    action = _actions->addAction( QString::fromLatin1("annotationdialog-rotate-right"),  _preview, SLOT( rotateRight() ) );
     action->setText(  i18n("Rotate clockwise") );
 
-    connect( _nextBut, SIGNAL( clicked() ), this, SLOT( slotNext() ) );
-    connect( _prevBut, SIGNAL( clicked() ), this, SLOT( slotPrev() ) );
-    connect( _rotateLeft, SIGNAL( clicked() ), this, SLOT( rotateLeft() ) );
-    connect( _rotateRight, SIGNAL( clicked() ), this, SLOT( rotateRight() ) );
-
     foreach (QAction* action, _actions->actions()) {
       action->setShortcutContext(Qt::WindowShortcut);
       addAction(action);
@@ -1045,4 +911,77 @@ void AnnotationDialog::Dialog::slotRatingChanged( unsigned int )
     _ratingChanged = true;
 }
 
+void AnnotationDialog::Dialog::continueLater()
+{
+    saveAndClose();
+}
+
+void AnnotationDialog::Dialog::saveAndClose()
+{
+    if (_origList.isEmpty()) {
+        // all images are deleted.
+        QDialog::accept();
+        return;
+    }
+
+    // I need to check for the changes first, as the case for _setup == InputSingleImageConfigMode, saves to the _origList,
+    // and we can thus not check for changes anymore
+    const bool anyChanges = hasChanges();
+
+    if ( _setup == InputSingleImageConfigMode )  {
+        writeToInfo();
+        for ( int i = 0; i < _editList.count(); ++i )  {
+            *(_origList[i]) = _editList[i];
+        }
+    }
+    else if ( _setup == InputMultiImageConfigMode ) {
+        for( Q3PtrListIterator<ListSelect> it( _optionList ); *it; ++it ) {
+            (*it)->slotReturn();
+        }
+
+        for( DB::ImageInfoListConstIterator it = _origList.constBegin(); it != _origList.constEnd(); ++it ) {
+            DB::ImageInfoPtr info = *it;
+            info->delaySavingChanges(true);
+            info->rotate( _preview->angle() );
+            if ( !_startDate->date().isNull() )
+                info->setDate( DB::ImageDate( _startDate->date(), _endDate->date(), _time->time() ) );
+
+            for( Q3PtrListIterator<ListSelect> listSelectIt( _optionList ); *listSelectIt; ++listSelectIt ) {
+                info->addCategoryInfo( (*listSelectIt)->category(), (*listSelectIt)->itemsOn() );
+                info->removeCategoryInfo( (*listSelectIt)->category(), (*listSelectIt)->itemsOff() );
+            }
+
+            if ( !_imageLabel->text().isEmpty() ) {
+                info->setLabel( _imageLabel->text() );
+            }
+
+
+            if ( !_description->toPlainText().isEmpty() ) {
+                info->setDescription( _description->toPlainText() );
+            }
+
+#ifdef HAVE_NEPOMUK
+            if( _ratingChanged)
+            {
+              info->setRating( _rating->rating() );
+            }
+#endif
+
+            info->delaySavingChanges(false);
+        }
+#ifdef HAVE_NEPOMUK
+        _ratingChanged = false;
+#endif
+    }
+    _accept = QDialog::Accepted;
+
+    // I shouldn't emit changed before I've actually commited the changes, otherwise the listeners will act on the old data.
+    if ( anyChanges ) {
+        MainWindow::DirtyIndicator::markDirty();
+        _thumbnailTextShouldReload = true;
+    }
+
+    QDialog::accept();
+}
+
 #include "Dialog.moc"
