@@ -15,15 +15,12 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
-#include "Exif/Database.h"
+#include "Database.h"
 
 #include "DB/ImageDB.h"
-#include "DB/ResultId.h"
-#include "Database.h"
 #include "Exif/DatabaseElement.h"
 #include "MainWindow/Window.h"
 #include "Settings/SettingsData.h"
-#include "Utilities/Util.h"
 
 #include <exiv2/exif.hpp>
 #include <exiv2/image.hpp>
@@ -32,7 +29,6 @@
 #include <kmessagebox.h>
 
 #include <QApplication>
-#include <QDebug>
 #include <QDir>
 #include <QFile>
 #include <QProgressDialog>
@@ -82,8 +78,8 @@ static void showError( QSqlQuery& query )
              "The error is likely due to a broken database file.</p>"
              "<p>To fix this problem run Maintainance->Rebuild EXIF database.</p>"
              "<hr/>"
-             "<p>For debugging, this is the command I tried to execute:<br/>%1</p>"
-             "<p>This was the error message I got:<br/>%2</p>",
+             "<p>For debugging: the command that was attempted to be executed was:<br/>%1</p>"
+             "<p>The error message obtained was:<br/>%2</p>",
              query.lastQuery(), query.lastError().text() );
 
     KMessageBox::information( MainWindow::Window::theMainWindow(), txt, i18n("Error Executing Exif Command"), QString::fromLatin1( "sql_error_in_exif_DB" )
@@ -95,13 +91,12 @@ static void showError( QSqlQuery& query )
 Exif::Database::Database()
     : _isOpen(false)
 {
+    _db = QSqlDatabase::addDatabase( QString::fromLatin1( "QSQLITE" ), QString::fromLatin1( "exif" ) );
 }
 
 
 void Exif::Database::openDatabase()
 {
-    _db = QSqlDatabase::addDatabase( QString::fromLatin1( "QSQLITE" ), QString::fromLatin1( "exif" ) );
-
     _db.setDatabaseName( exifDBFile() );
 
     if ( !_db.open() )
@@ -158,6 +153,7 @@ bool Exif::Database::add( const QString& fileName )
     }
     catch (...)
     {
+        qWarning("Error while reading exif information from %s", qPrintable(fileName) );
         return false;
     }
 }
@@ -299,8 +295,9 @@ void Exif::Database::recreate()
     // the user presse 'cancel' or there is any error. In that case
     // we want to go back to the original DB.
 
-    QString origBackup = exifDBFile() + QLatin1String(".bak");
+    const QString origBackup = exifDBFile() + QLatin1String(".bak");
     _db.close();
+
     QDir().remove(origBackup);
     QDir().rename(exifDBFile(), origBackup);
     init();
