@@ -15,23 +15,19 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
-#include "Exif/SearchDialog.h"
+#include "SearchDialog.h"
 #include <klocale.h>
 #include <qlayout.h>
 #include <q3vgroupbox.h>
 #include <qcheckbox.h>
-//Added by qt3to4:
 #include <QHBoxLayout>
 #include <QVBoxLayout>
 #include "Exif/Database.h"
 #include <q3grid.h>
-#include "SearchDialog.h"
 
 #include <qlabel.h>
 #include <qspinbox.h>
-#include <kvbox.h>
 #include <QScrollArea>
-#include <kdebug.h>
 
 using namespace Exif;
 
@@ -77,7 +73,7 @@ Exif::SearchDialog::SearchDialog( QWidget* parent )
     _toFocalLength->setSingleStep( 10 );
 
     _toFocalLength->setValue( 10000 );
-    QString suffix = i18nc( "This is milimeter for focal length, like 35mm", "mm" );
+    QString suffix = i18nc( "This is millimeter for focal length, like 35mm", "mm" );
     _fromFocalLength->setSuffix( suffix );
     _toFocalLength->setSuffix( suffix );
 
