@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2009 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -17,6 +17,9 @@
 */
 
 #include "Window.h"
+#include "ThumbnailView/ThumbnailFacade.h"
+#include <KActionCollection>
+#include "BreadcrumbViewer.h"
 #include <QDebug>
 
 #include "StatisticsDialog.h"
@@ -30,7 +33,6 @@
 #include <QCloseEvent>
 #include <QVBoxLayout>
 #include <Q3Frame>
-#include "ThumbnailView/ThumbnailWidget.h"
 #include "ThumbnailView/ThumbnailBuilder.h"
 #include "AnnotationDialog/Dialog.h"
 #include <qdir.h>
@@ -58,7 +60,7 @@
 #include "DeleteDialog.h"
 #include <ksimpleconfig.h>
 #include <kcmdlineargs.h>
-#include <q3popupmenu.h>
+#include <QMenu>
 #include <kiconloader.h>
 #include <kpassworddialog.h>
 #include <KShortcutsDialog>
@@ -124,6 +126,7 @@
 #include <stdexcept>
 #include <KInputDialog>
 #include "DB/Result.h"
+#include "ThumbnailView/enums.h"
 
 MainWindow::Window* MainWindow::Window::_instance = 0;
 
@@ -156,27 +159,10 @@ MainWindow::Window::Window( QWidget* parent )
     lay->addWidget( line );
 
     _browser = new Browser::BrowserWidget( _stack );
-    connect( _browser, SIGNAL( showingOverview() ), this, SLOT( showBrowser() ) );
-    connect( _browser, SIGNAL( pathChanged( const QString& ) ), this, SLOT( pathChanged( const QString& ) ) );
-    connect( _browser, SIGNAL( pathChanged( const QString& ) ), this, SLOT( updateDateBar( const QString& ) ) );
-    _thumbnailView = new ThumbnailView::ThumbnailWidget( _stack );
-    connect( _dateBar, SIGNAL( dateSelected( const DB::ImageDate&, bool ) ), _thumbnailView, SLOT( gotoDate( const DB::ImageDate&, bool ) ) );
-    connect( _dateBar, SIGNAL( toolTipInfo( const QString& ) ), this, SLOT( showDateBarTip( const QString& ) ) );
-    connect( Settings::SettingsData::instance(), SIGNAL( histogramSizeChanged( const QSize& ) ), _dateBar, SLOT( setHistogramBarSize( const QSize& ) ) );
-
-
-    connect( _dateBar, SIGNAL( dateRangeChange( const DB::ImageDate& ) ),
-             this, SLOT( setDateRange( const DB::ImageDate& ) ) );
-    connect( _dateBar, SIGNAL( dateRangeCleared() ), this, SLOT( clearDateRange() ) );
-
-    connect( _thumbnailView, SIGNAL( showImage( const DB::ResultId& ) ), this, SLOT( showImage( const DB::ResultId& ) ) );
-    connect( _thumbnailView, SIGNAL( showSelection() ), this, SLOT( slotView() ) );
-    connect( _thumbnailView, SIGNAL( currentDateChanged( const QDateTime& ) ), _dateBar, SLOT( setDate( const QDateTime& ) ) );
-
-    connect( _thumbnailView, SIGNAL( fileNameUnderCursorChanged( const QString& ) ), this, SLOT( slotSetFileName( const QString& ) ) );
+    _thumbnailView = new ThumbnailView::ThumbnailFacade();
 
     _stack->addWidget( _browser );
-    _stack->addWidget( _thumbnailView );
+    _stack->addWidget( _thumbnailView->gui() );
     _stack->raiseWidget( _browser );
 
     _optionsDialog = 0;
@@ -190,18 +176,34 @@ MainWindow::Window::Window( QWidget* parent )
     connect( _autoSaveTimer, SIGNAL( timeout() ), this, SLOT( slotAutoSave() ) );
     startAutoSaveTimer();
 
+    connect( _browser, SIGNAL( showingOverview() ), this, SLOT( showBrowser() ) );
+    connect( _browser, SIGNAL( pathChanged( const Browser::BreadcrumbList& ) ), _pathIndicator, SLOT( setBreadcrumbs( const Browser::BreadcrumbList& ) ) );
+    connect( _pathIndicator, SIGNAL( widenToBreadcrumb( const Browser::Breadcrumb& ) ), _browser, SLOT( widenToBreadcrumb( const Browser::Breadcrumb& ) ) );
+    connect( _browser, SIGNAL( pathChanged( const Browser::BreadcrumbList& ) ), this, SLOT( updateDateBar( const Browser::BreadcrumbList& ) ) );
+    connect( _dateBar, SIGNAL( dateSelected( const DB::ImageDate&, bool ) ), _thumbnailView, SLOT( gotoDate( const DB::ImageDate&, bool ) ) );
+    connect( _dateBar, SIGNAL( toolTipInfo( const QString& ) ), this, SLOT( showDateBarTip( const QString& ) ) );
+    connect( Settings::SettingsData::instance(), SIGNAL( histogramSizeChanged( const QSize& ) ), _dateBar, SLOT( setHistogramBarSize( const QSize& ) ) );
+
+
+    connect( _dateBar, SIGNAL( dateRangeChange( const DB::ImageDate& ) ),
+             this, SLOT( setDateRange( const DB::ImageDate& ) ) );
+    connect( _dateBar, SIGNAL( dateRangeCleared() ), this, SLOT( clearDateRange() ) );
+
+    connect( _thumbnailView, SIGNAL( showImage( const DB::ResultId& ) ), this, SLOT( showImage( const DB::ResultId& ) ) );
+    connect( _thumbnailView, SIGNAL( showSelection() ), this, SLOT( slotView() ) );
+    connect( _thumbnailView, SIGNAL( currentDateChanged( const QDateTime& ) ), _dateBar, SLOT( setDate( const QDateTime& ) ) );
+
+    connect( _thumbnailView, SIGNAL( fileIdUnderCursorChanged( const DB::ResultId& ) ), this, SLOT( slotSetFileName( const DB::ResultId& ) ) );
     connect( DB::ImageDB::instance(), SIGNAL( totalChanged( uint ) ), this, SLOT( updateDateBar() ) );
     connect( DB::ImageDB::instance(), SIGNAL( dirty() ), _dirtyIndicator, SLOT( markDirtySlot() ) );
     connect( DB::ImageDB::instance()->categoryCollection(), SIGNAL( categoryCollectionChanged() ), this, SLOT( slotOptionGroupChanged() ) );
-    connect( _browser, SIGNAL( showingOverview() ), _partial, SLOT( showingOverview() ) );
-    connect( _thumbnailView, SIGNAL( selectionChanged() ), this, SLOT( slotThumbNailSelectionChanged() ) );
+    connect( _browser, SIGNAL( imageCount(uint)), _partial, SLOT( showBrowserMatches(uint) ) );
+    connect( _thumbnailView, SIGNAL( selectionChanged(int) ), this, SLOT( slotThumbNailSelectionChanged(int) ) );
 
     connect( _dirtyIndicator, SIGNAL( dirty() ), _thumbnailView, SLOT(repaintScreen() ) );
 
-    statusBar()->showMessage(i18n("Welcome to KPhotoAlbum"), 5000 );
-
     QTimer::singleShot( 0, this, SLOT( delayedInit() ) );
-    slotThumbNailSelectionChanged();
+    slotThumbNailSelectionChanged(0);
 
     // Automatically save toolbar settings
     setAutoSaveSettings();
@@ -324,7 +326,7 @@ void MainWindow::Window::slotCreateImageStack()
             DB::ImageDB::instance()->unstack( list );
             if ( ! DB::ImageDB::instance()->stack( list ) ) {
                 KMessageBox::sorry( this,
-                        i18n("Something fishy happened, stack creation failed, sorry."),
+                        i18n("Unknown error, stack creation failed."),
                         i18n("Stacking Error"));
                 return;
             }
@@ -473,16 +475,13 @@ void MainWindow::Window::slotDeleteSelected()
 
 void MainWindow::Window::slotCopySelectedURLs()
 {
-    QStringList res;
-    QList<QUrl> urls;
+    KUrl::List urls;
     Q_FOREACH(const DB::ImageInfoPtr info, selectedOnDisk().fetchInfos()) {
         const QString fileName = info->fileName(DB::AbsolutePath);
-        res.append( fileName );
-        urls.append( QUrl( fileName ) );
+        urls.append( fileName );
     }
     QMimeData* mimeData = new QMimeData;
-    mimeData->setUrls( urls );
-    mimeData->setText( res.join( QLatin1String(" ") ) );
+    urls.populateMimeData(mimeData);
 
     QApplication::clipboard()->setMimeData( mimeData );
 }
@@ -502,7 +501,7 @@ void MainWindow::Window::slotReReadExifInfo()
 
 DB::Result MainWindow::Window::selected(bool keepSortOrderOfDatabase)
 {
-    if ( _thumbnailView == _stack->visibleWidget() )
+    if ( _thumbnailView->gui() == _stack->visibleWidget() )
         return _thumbnailView->selection( keepSortOrderOfDatabase );
     else
         return DB::Result();
@@ -543,12 +542,12 @@ void MainWindow::Window::launchViewer(const DB::Result& inputMediaList, bool reu
     DB::Result mediaList = inputMediaList;
     int seek = -1;
     if (mediaList.isEmpty()) {
-        mediaList = _thumbnailView->imageList( ThumbnailView::ThumbnailWidget::ViewOrder );
+        mediaList = _thumbnailView->imageList( ThumbnailView::ViewOrder );
     } else if (mediaList.size() == 1) {
         // we fake it so it appears the user has selected all images
         // and magically scrolls to the originally selected one
         DB::ResultId first = mediaList.at(0);
-        mediaList = _thumbnailView->imageList( ThumbnailView::ThumbnailWidget::ViewOrder );
+        mediaList = _thumbnailView->imageList( ThumbnailView::ViewOrder );
         seek = mediaList.indexOf(first);
     }
 
@@ -594,7 +593,7 @@ void MainWindow::Window::slotSortByDateAndTime()
 
 QString MainWindow::Window::welcome()
 {
-    WelComeDialog dialog( this );
+    WelcomeDialog dialog( this );
     dialog.exec();
     return dialog.configFileName();
 }
@@ -808,36 +807,37 @@ void MainWindow::Window::setupMenuBar()
     a->setText( i18n("Enable All Messages") );
 
     _viewMenu = actionCollection()->add<KActionMenu>( QString::fromLatin1("configureView") );
-    _viewMenu->setText( i18n("Configure View") );
+    _viewMenu->setText( i18n("Configure Current View") );
 
     _viewMenu->setIcon( KIcon( QString::fromLatin1( "view-list-details" ) ) );
     _viewMenu->setDelayed( false );
-    connect( _browser, SIGNAL( showsContentView( bool ) ), _viewMenu, SLOT( setEnabled( bool ) ) );
 
     QActionGroup* viewGrp = new QActionGroup( this );
     viewGrp->setExclusive( true );
 
     _smallListView = actionCollection()->add<KToggleAction>( QString::fromLatin1("smallListView"), _browser, SLOT( slotSmallListView() ) );
-    _smallListView->setText( i18n("List View") );
+    _smallListView->setText( i18n("Tree") );
     _viewMenu->addAction( _smallListView );
     _smallListView->setActionGroup( viewGrp );
 
     _largeListView = actionCollection()->add<KToggleAction>( QString::fromLatin1("largelistview"), _browser, SLOT( slotLargeListView() ) );
-    _largeListView->setText( i18n("List View with Custom Icons") );
+    _largeListView->setText( i18n("Tree with User Icons") );
     _viewMenu->addAction( _largeListView );
     _largeListView->setActionGroup( viewGrp );
 
+#if 0 // I see no reason for this one.
     _smallIconView = actionCollection()->add<KToggleAction>( QString::fromLatin1("smalliconview"),  _browser, SLOT( slotSmallIconView() ) );
-    _smallIconView->setText( i18n("Icon View") );
+    _smallIconView->setText( i18n("Icons") );
     _viewMenu->addAction( _smallIconView );
     _smallIconView->setActionGroup( viewGrp );
+#endif
 
     _largeIconView = actionCollection()->add<KToggleAction>(  QString::fromLatin1("largeiconview"), _browser, SLOT( slotLargeIconView() ) );
-    _largeIconView->setText( i18n("Icon View with Custom Icons") );
+    _largeIconView->setText( i18n("Icons") );
     _viewMenu->addAction( _largeIconView );
     _largeIconView->setActionGroup( viewGrp );
 
-    connect( _browser, SIGNAL( browsingInSomeCategory( bool ) ), viewGrp, SLOT( setEnabled( bool ) ) );
+    connect( _browser, SIGNAL( isViewChangeable( bool ) ), viewGrp, SLOT( setEnabled( bool ) ) );
 
     connect( _browser, SIGNAL( currentViewTypeChanged( DB::Category::ViewType ) ),
              this, SLOT( slotUpdateViewMenu( DB::Category::ViewType ) ) );
@@ -864,7 +864,7 @@ void MainWindow::Window::setupMenuBar()
     _showExifDialog = actionCollection()->addAction( QString::fromLatin1("showExifInfo"), this, SLOT( slotShowExifInfo() ) );
     _showExifDialog->setText( i18n("Show Exif Info") );
 #endif
-    _recreateThumbnails = actionCollection()->addAction( QString::fromLatin1("recreateThumbnails"), this, SLOT( slotRecreateThumbnail() ) );
+    _recreateThumbnails = actionCollection()->addAction( QString::fromLatin1("recreateThumbnails"), _thumbnailView, SLOT( slotRecreateThumbnail() ) );
     _recreateThumbnails->setText( i18n("Recreate Selected Thumbnails") );
 
     createGUI( QString::fromLatin1( "kphotoalbumui.rc" ) );
@@ -901,13 +901,14 @@ void MainWindow::Window::slotAutoSave()
 void MainWindow::Window::showThumbNails()
 {
     reloadThumbnails(false);
-    _stack->raiseWidget( _thumbnailView );
-    _thumbnailView->setFocus();
+    _stack->raiseWidget( _thumbnailView->gui() );
+    _thumbnailView->gui()->setFocus();
     updateStates( true );
 }
 
 void MainWindow::Window::showBrowser()
 {
+    statusBar()->clearMessage();
     _stack->raiseWidget( _browser );
     _browser->setFocus();
     updateStates( false );
@@ -927,15 +928,6 @@ void MainWindow::Window::showTipOfDay()
     KTipDialog::showTip( this, QString::null, true );
 }
 
-void MainWindow::Window::pathChanged( const QString& path )
-{
-    QString text = path;
-
-    if ( text.length() > 80 )
-        text = text.left(80) + QString::fromLatin1( "..." );
-
-    _pathIndicator->setText( text );
-}
 
 void MainWindow::Window::runDemo()
 {
@@ -1031,8 +1023,8 @@ bool MainWindow::Window::load()
 
 void MainWindow::Window::contextMenuEvent( QContextMenuEvent* e )
 {
-    if ( _stack->visibleWidget() == _thumbnailView ) {
-        Q3PopupMenu menu( this, "context popup menu");
+    if ( _stack->visibleWidget() == _thumbnailView->gui() ) {
+        QMenu menu( this );
         menu.addAction( _configOneAtATime );
         menu.addAction( _configAllSimultaniously );
         menu.addAction( _createImageStack );
@@ -1185,15 +1177,16 @@ void MainWindow::Window::slotConfigureKeyBindings()
     delete viewer;
 }
 
-void MainWindow::Window::slotSetFileName( const QString& fileName )
+void MainWindow::Window::slotSetFileName( const DB::ResultId& id )
 {
-    statusBar()->showMessage( fileName, 4000 );
+    if ( id.isNull() )
+        statusBar()->clearMessage();
+    else
+        statusBar()->showMessage( id.fetchInfo()->fileName(DB::AbsolutePath), 4000 );
 }
 
-void MainWindow::Window::slotThumbNailSelectionChanged()
+void MainWindow::Window::slotThumbNailSelectionChanged(int selectionSize)
 {
-    const int selectionSize = _thumbnailView->selection().size();
-
     _configAllSimultaniously->setEnabled(selectionSize > 1);
     _configOneAtATime->setEnabled(selectionSize >= 1);
     _createImageStack->setEnabled(selectionSize > 1);
@@ -1233,7 +1226,7 @@ void MainWindow::Window::slotRotateSelectedRight()
 void MainWindow::Window::reloadThumbnails(bool flushCache)
 {
     _thumbnailView->reload( flushCache );
-    slotThumbNailSelectionChanged();
+    slotThumbNailSelectionChanged( _thumbnailView->selection().size() );
 }
 
 void MainWindow::Window::reloadThumbnailsAndFlushCache()
@@ -1248,12 +1241,14 @@ void MainWindow::Window::reloadThumbnailsAfterRotation()
 
 void MainWindow::Window::slotUpdateViewMenu( DB::Category::ViewType type )
 {
-    if ( type == DB::Category::ListView )
+    if ( type == DB::Category::TreeView )
         _smallListView->setChecked( true );
-    else if ( type == DB::Category::ThumbedListView )
+    else if ( type == DB::Category::ThumbedTreeView )
         _largeListView->setChecked( true );
+#if 0
     else if ( type == DB::Category::IconView )
         _smallIconView->setChecked( true );
+#endif
     else if ( type == DB::Category::ThumbedIconView )
         _largeIconView->setChecked( true );
 }
@@ -1334,7 +1329,7 @@ void MainWindow::Window::slotExport()
 
 void MainWindow::Window::slotReenableMessages()
 {
-    int ret = KMessageBox::questionYesNo( this, i18n("<p>Really enable all messageboxes where you previously "
+    int ret = KMessageBox::questionYesNo( this, i18n("<p>Really enable all message boxes where you previously "
                                                      "checked the do-not-show-again check box?</p>" ) );
     if ( ret == KMessageBox::Yes )
         KMessageBox::enableAllMessages();
@@ -1344,7 +1339,12 @@ void MainWindow::Window::slotReenableMessages()
 void MainWindow::Window::setupPluginMenu()
 {
     QMenu* menu = findChild<QMenu*>( QString::fromLatin1("plugins") );
-    Q_ASSERT( menu );
+    if ( !menu ) {
+        KMessageBox::error( this, i18n("<p>KPhotoAlbum hit an internal error (missing plug-in menu in MainWindow::Window::setupPluginMenu). This indicate that you forgot to do a make install. If you did compile KPhotoAlbum yourself, then please run make install. If not, please report this as a bug.</p><p>KPhotoAlbum will continue execution, but it is not entirely unlikely that it will crash later on due to the missing make install.</p>" ), i18n("Internal Error") );
+        _hasLoadedPlugins = true;
+        return; // This is no good, but lets try and continue.
+    }
+
 
 #ifdef HASKIPI
     connect( menu, SIGNAL( aboutToShow() ), this, SLOT( loadPlugins() ) );
@@ -1374,13 +1374,13 @@ void MainWindow::Window::loadPlugins()
     _pluginLoader->loadPlugins();
 
     // Setup signals
-    connect( _thumbnailView, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged() ) );
+    connect( _thumbnailView, SIGNAL( selectionChanged(int) ), this, SLOT( slotSelectionChanged(int) ) );
     _hasLoadedPlugins = true;
 
     // Make sure selection is updated also when plugin loading is
     // delayed. This is needed, because selection might already be
     // non-empty when loading the plugins.
-    slotSelectionChanged();
+    slotSelectionChanged(selected().size());
 #endif // HASKIPI
 }
 
@@ -1478,10 +1478,10 @@ QString MainWindow::Window::currentBrowseCategory() const
     return _browser->currentCategory();
 }
 
-void MainWindow::Window::slotSelectionChanged()
+void MainWindow::Window::slotSelectionChanged( int count )
 {
 #ifdef HASKIPI
-    _pluginInterface->slotSelectionChanged( !selected().isEmpty() );
+    _pluginInterface->slotSelectionChanged( count != 0 );
 #endif
 }
 
@@ -1527,12 +1527,12 @@ void MainWindow::Window::slotShowListOfFiles()
         showThumbNails( out );
 }
 
-void MainWindow::Window::updateDateBar( const QString& path )
+void MainWindow::Window::updateDateBar( const Browser::BreadcrumbList& path )
 {
     static QString lastPath = QString::fromLatin1("ThisStringShouldNeverBeSeenSoWeUseItAsInitialContent");
-    if ( path != lastPath )
+    if ( path.toString() != lastPath )
         updateDateBar();
-    lastPath = path;
+    lastPath = path.toString();
 }
 
 void MainWindow::Window::updateDateBar()
@@ -1681,20 +1681,6 @@ void MainWindow::Window::slotOrderDecr()
     _thumbnailView->setSortDirection( ThumbnailView::NewestFirst );
 }
 
-void MainWindow::Window::slotRecreateThumbnail()
-{
-    Q_FOREACH(const DB::ImageInfoPtr info, selectedOnDisk().fetchInfos()) {
-        QString fileName = info->fileName(DB::AbsolutePath);
-        ImageManager::Manager::instance()->removeThumbnail( fileName );
-
-        int size = Settings::SettingsData::instance()->thumbSize();
-        ImageManager::ImageRequest* request = new ImageManager::ImageRequest( fileName, QSize(size,size), info->angle(), _thumbnailView );
-        request->setPriority( ImageManager::BatchTask );
-        ImageManager::Manager::instance()->load( request );
-    }
-}
-
-
 void MainWindow::Window::showVideos()
 {
     KRun::runUrl(KUrl(QString::fromLatin1("http://www.kphotoalbum.org/videos.html")), QString::fromLatin1( "text/html" ), this );
@@ -1731,8 +1717,8 @@ void MainWindow::Window::setupStatusBar()
     total->setTotal( DB::ImageDB::instance()->totalCount() );
     connect( DB::ImageDB::instance(), SIGNAL( totalChanged( uint ) ), total, SLOT( setTotal( uint ) ) );
 
-    _pathIndicator = new QLabel;
-    statusBar()->addWidget( _pathIndicator );
+    _pathIndicator = new BreadcrumbViewer;
+    statusBar()->addWidget( _pathIndicator, 1 );
 }
 
 void MainWindow::Window::slotRecreateExifDB()
@@ -1747,13 +1733,13 @@ void MainWindow::Window::createSarchBar()
     // Set up the search tool bar
     SearchBar* bar = new SearchBar( this );
     bar->setLineEditEnabled(false);
+    bar->setObjectName( QString::fromAscii("searchBar" ) );
 
     connect( bar, SIGNAL( textChanged( const QString& ) ), _browser, SLOT( slotLimitToMatch( const QString& ) ) );
     connect( bar, SIGNAL( returnPressed() ), _browser, SLOT( slotInvokeSeleted() ) );
-    connect( bar, SIGNAL( scrollLine( int ) ), _browser, SLOT( scrollLine( int ) ) );
-    connect( bar, SIGNAL( scrollPage( int ) ), _browser, SLOT( scrollPage( int ) ) );
+    connect( bar, SIGNAL( keyPressed( QKeyEvent* ) ), _browser, SLOT( scrollKeyPressed( QKeyEvent* ) ) );
     connect( _browser, SIGNAL( viewChanged() ), bar, SLOT( reset() ) );
-    connect( _browser, SIGNAL( showsContentView( bool ) ), bar, SLOT( setLineEditEnabled( bool ) ) );
+    connect( _browser, SIGNAL( isSearchable( bool ) ), bar, SLOT( setLineEditEnabled( bool ) ) );
 }
 
 #include "Window.moc"
