@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2006 Jesper K. Pedersen <blackie@kde.org>
+/* Copyright (C) 2003-2009 Jesper K. Pedersen <blackie@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -18,22 +18,28 @@
 #ifndef MOUSETRACKINGINTERACTION_H
 #define MOUSETRACKINGINTERACTION_H
 
+#include "ThumbnailComponent.h"
 #include "MouseInteraction.h"
-//Added by qt3to4:
 #include <QMouseEvent>
 
 namespace ThumbnailView
 {
-class ThumbnailWidget;
-
-class MouseTrackingInteraction : public MouseInteraction {
+class MouseTrackingInteraction : public QObject, public MouseInteraction, private ThumbnailComponent {
+    Q_OBJECT
 
 public:
-    MouseTrackingInteraction( ThumbnailWidget* );
+    MouseTrackingInteraction( ThumbnailFactory* factory );
     virtual void mouseMoveEvent( QMouseEvent* );
 
+signals:
+    void fileIdUnderCursorChanged( const DB::ResultId& id );
+
 private:
-    ThumbnailWidget* _view;
+    void updateStackingIndication( QMouseEvent* event );
+    void handleCursorOverNewIcon( QMouseEvent* event );
+
+private:
+    bool _cursorWasAtStackIcon;
 };
 
 }
