@@ -19,7 +19,6 @@
 
 #include "Viewer/ImageDisplay.h"
 #include <qpainter.h>
-//Added by qt3to4:
 #include <QPaintEvent>
 #include <QResizeEvent>
 #include <QMouseEvent>
@@ -276,6 +275,11 @@ void Viewer::ImageDisplay::zoomIn()
 void Viewer::ImageDisplay::zoomOut()
 {
     QPoint size = (_zEnd-_zStart);
+
+    //Bug 150971, Qt tries to render bigger and bigger images (10000x10000), hence running out of memory.
+    if ( ( size.x() * size.y() > 25*1024*1024 ) )
+        return;
+
     QPoint p1 = _zStart - size*(0.25/2);
     QPoint p2 = _zEnd + size*(0.25/2);
     zoom(p1,p2);
@@ -365,7 +369,7 @@ void Viewer::ImageDisplay::pixmapLoaded( const QString& fileName, const QSize& i
     if ( loadedOK && fileName == _info->fileName(DB::AbsolutePath) ) {
         if ( fullSize.isValid() && !_info->size().isValid() )
             _info->setSize( fullSize );
-            
+
         if ( !_reloadImageInProgress )
             updateZoomPoints( Settings::SettingsData::instance()->viewerStandardSize(), img.size() );
         else {
@@ -529,7 +533,9 @@ void Viewer::ImageDisplay::updateZoomPoints( const Settings::StandardViewSize ty
 
 void Viewer::ImageDisplay::potentialyLoadFullSize()
 {
-    if ( _info->size() != _loadedImage.size() ) {
+    // The second part of this disables loading a higher resolution. The reason for this is that zooming in a rotated image is broken, and we can't find where,
+    // so this is a work around to get zooming to work at all.
+    if ( _info->size() != _loadedImage.size() && _info->angle() == 0 ) {
         ImageManager::ImageRequest* request = new ImageManager::ImageRequest( _info->fileName(DB::AbsolutePath), QSize(-1,-1), _info->angle(), this );
         request->setPriority( ImageManager::Viewer );
         ImageManager::Manager::instance()->load( request );
