@@ -35,6 +35,7 @@
 #include "MainWindow/Window.h"
 #include "CategoryImageConfig.h"
 #include "MainWindow/ExternalPopup.h"
+#include "MainWindow/CategoryImagePopup.h"
 #include "DB/CategoryCollection.h"
 #include "DB/ImageDB.h"
 #include "InfoBox.h"
@@ -63,21 +64,6 @@
 #  include "Exif/InfoDialog.h"
 #endif
 
-/**
- * \namespace Viewer
- * \brief Viewer used for displaying images and videos
- *
- * This class implements the viewer used to display images and videos.
- *
- * The class consists of these components:
- * <ul>
- * <li>\ref ViewerWidget - This is the topmost widget used as the viewer.
- * <li>\ref Display, \ref ImageDisplay, \ref VideoDisplay and \ref TextDisplay - Widgets hierarchy which takes care of the actual displaying of content.
- * <li> \ref ViewHandler - Handler which interprets mouse gestures.
- * <li> \ref InfoBox - Widget implementing the informatiom box
- * <li> \ref SpeedDisplay - Widget implementing the toplevel display used when adjusting slideshow speed.
- * </ul>
- */
 Viewer::ViewerWidget* Viewer::ViewerWidget::_latest = 0;
 
 Viewer::ViewerWidget* Viewer::ViewerWidget::latest()
@@ -149,6 +135,7 @@ void Viewer::ViewerWidget::setupContextMenu()
     createWallPaperMenu();
     createInvokeExternalMenu();
     createVideoMenu();
+    createCategoryImageMenu();
 
     KAction* action = _actions->addAction( QString::fromLatin1("viewer-edit-image-properties"), this, SLOT( editImage() ) );
     action->setText( i18n("Annotate...") );
@@ -160,11 +147,6 @@ void Viewer::ViewerWidget::setupContextMenu()
     _setStackHead->setShortcut( Qt::CTRL+Qt::Key_4 );
     _popup->addAction( _setStackHead );
 
-    // PENDING(blackie) This should only be enabled for image displays.
-    _categoryEditor = _actions->addAction( QString::fromLatin1("viewer-show-category-editor"), this, SLOT( makeCategoryImage() ) );
-    _categoryEditor->setText( i18n("Show Category Editor") );
-    _popup->addAction(_categoryEditor);
-
 #ifdef HAVE_EXIV2
     _showExifViewer = _actions->addAction( QString::fromLatin1("viewer-show-exif-viewer"), this, SLOT( showExifViewer() ) );
     _showExifViewer->setText( i18n("Show EXIF Viewer") );
@@ -329,6 +311,11 @@ void Viewer::ViewerWidget::createSkipMenu()
     popup->addAction( action );
     _backwardActions.append(action);
 
+    action = _actions->addAction( QString::fromLatin1("viewer-remove-current"), this, SLOT( removeCurrent() ) );
+    action->setText( i18n("Remove Image from Display List") );
+    action->setShortcut( Qt::Key_Delete );
+    popup->addAction( action );
+
     _popup->addMenu( popup );
 }
 
@@ -426,7 +413,7 @@ void Viewer::ViewerWidget::load()
 
     _rotateMenu->setEnabled( !isVideo );
     _wallpaperMenu->setEnabled( !isVideo );
-    _categoryEditor->setEnabled( !isVideo );
+    _categoryImagePopup->setEnabled( !isVideo );
 #ifdef HAVE_EXIV2
     _showExifViewer->setEnabled( !isVideo );
 #endif
@@ -491,10 +478,10 @@ void Viewer::ViewerWidget::showNextN(int n)
         _videoDisplay->stop();
     }
 
-    if ( _current +1 < (int) _list.count() )  {
+    if ( _current + n < (int) _list.count() )  {
         _current += n;
-	if (_current >= (int) _list.count())
-	  _current = (int) _list.count() - 1;
+        if (_current >= (int) _list.count())
+            _current = (int) _list.count() - 1;
         _forward = true;
         load();
     }
@@ -505,6 +492,18 @@ void Viewer::ViewerWidget::showNext()
     showNextN(1);
 }
 
+void Viewer::ViewerWidget::removeCurrent()
+{
+    _list.removeAll(_list[_current]);
+    qDebug("%d,%d", _current, _list.count() );
+    if ( _current == _list.count() ) {
+        qDebug("Show prev");
+        showPrev();
+    }
+    else
+        showNextN(0);
+}
+
 void Viewer::ViewerWidget::showNext10()
 {
     showNextN(10);
@@ -866,12 +865,6 @@ void Viewer::ViewerWidget::setShowFullScreen( bool on )
     _showingFullScreen = on;
 }
 
-void Viewer::ViewerWidget::makeCategoryImage()
-{
-    CategoryImageConfig::instance()->setCurrentImage( _imageDisplay->currentViewAsThumbnail(), currentInfo() );
-    CategoryImageConfig::instance()->show();
-}
-
 void Viewer::ViewerWidget::updateCategoryConfig()
 {
     if ( !CategoryImageConfig::instance()->isVisible() )
@@ -886,6 +879,11 @@ void Viewer::ViewerWidget::populateExternalPopup()
     _externalPopup->populate( currentInfo(), _list );
 }
 
+void Viewer::ViewerWidget::populateCategoryImagePopup()
+{
+    _categoryImagePopup->populate( _imageDisplay->currentViewAsThumbnail(), _list[_current] );
+}
+
 void Viewer::ViewerWidget::show( bool slideShow )
 {
     QSize size;
@@ -1001,8 +999,8 @@ void Viewer::ViewerWidget::createVideoMenu()
          << SeekInfo( i18n("1 minute backward"), "seek-1-minute", -60000, QKeySequence(QString::fromLatin1( "Shift+Left")))
          << SeekInfo( i18n("10 seconds backward"), "seek-10-second", -10000, QKeySequence(QString::fromLatin1( "Left")))
          << SeekInfo( i18n("1 seconds backward"), "seek-1-second", -1000, QKeySequence(QString::fromLatin1( "Up")))
-         << SeekInfo( i18n("100 miliseconds backward"), "seek-100-millisecond", -100, QKeySequence(QString::fromLatin1( "Shift+Up")))
-         << SeekInfo( i18n("100 miliseconds forward"), "seek+100-millisecond", 100, QKeySequence(QString::fromLatin1( "Shift+Down")))
+         << SeekInfo( i18n("100 milliseconds backward"), "seek-100-millisecond", -100, QKeySequence(QString::fromLatin1( "Shift+Up")))
+         << SeekInfo( i18n("100 milliseconds forward"), "seek+100-millisecond", 100, QKeySequence(QString::fromLatin1( "Shift+Down")))
          << SeekInfo( i18n("1 seconds forward"), "seek+1-second", 1000, QKeySequence(QString::fromLatin1( "Down")))
          << SeekInfo( i18n("10 seconds forward"), "seek+10-second", 10000, QKeySequence(QString::fromLatin1( "Right")))
          << SeekInfo( i18n("1 minute forward"), "seek+1-minute", 60000, QKeySequence(QString::fromLatin1( "Shift+Right")))
@@ -1045,6 +1043,13 @@ void Viewer::ViewerWidget::createVideoMenu()
     _videoActions.append( restart );
 }
 
+void Viewer::ViewerWidget::createCategoryImageMenu()
+{
+    _categoryImagePopup = new MainWindow::CategoryImagePopup( _popup );
+    _popup->addMenu( _categoryImagePopup );
+    connect( _categoryImagePopup, SIGNAL( aboutToShow() ), this, SLOT( populateCategoryImagePopup() ) );
+}
+
 void Viewer::ViewerWidget::test()
 {
 #ifdef TESTING
