@@ -17,11 +17,6 @@
 */
 #include "FileReader.h"
 
-#ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-#endif
-
-#include <iostream>
 #include <QTextCodec>
 #include <QTextStream>
 #include <klocale.h>
@@ -98,7 +93,7 @@ void XMLDB::FileReader::createSpecialCategories()
     DB::CategoryPtr folderCat = _db->_categoryCollection.categoryForName( QString::fromLatin1( "Folder" ) );
     if( folderCat.isNull() ) {
         folderCat = new XMLCategory( QString::fromLatin1("Folder"), QString::fromLatin1("folder"),
-                                     DB::Category::ListView, 32, false );
+                                     DB::Category::TreeView, 32, false );
         _db->_categoryCollection.addCategory( folderCat );
     }
     folderCat->setSpecialCategory( true );
@@ -107,7 +102,7 @@ void XMLDB::FileReader::createSpecialCategories()
     DB::CategoryPtr tokenCat = _db->_categoryCollection.categoryForName( QString::fromLatin1( "Tokens" ) );
     if ( !tokenCat ) {
         tokenCat = new XMLCategory( QString::fromLatin1("Tokens"), QString::fromLatin1("flag-blue"),
-                                    DB::Category::ListView, 32, true );
+                                    DB::Category::TreeView, 32, true );
         _db->_categoryCollection.addCategory( tokenCat );
     }
     tokenCat->setSpecialCategory( true );
@@ -120,7 +115,7 @@ void XMLDB::FileReader::createSpecialCategories()
     DB::CategoryPtr mediaCat = _db->_categoryCollection.categoryForName( QString::fromLatin1( "Media Type" ) );
     if ( !mediaCat ) {
         mediaCat = new XMLCategory( QString::fromLatin1("Media Type"), QString::fromLatin1("video"),
-                                    DB::Category::ListView, 32, false );
+                                    DB::Category::TreeView, 32, false );
         _db->_categoryCollection.addCategory( mediaCat );
     }
     mediaCat->addItem( QString::fromLatin1( "Image" ) );
@@ -192,10 +187,10 @@ void XMLDB::FileReader::loadImages( const QDomElement& images )
             continue;
 
         QString fileName = elm.attribute( QString::fromLatin1("file") );
-        if ( fileName.isNull() )
+        if ( fileName.isNull() ) {
             qWarning( "Element did not contain a file attribute" );
-        else if (_db->_idMapper.exists(fileName)) {
-            std::cout << fileName.toLatin1().data() << " already in database"<<std::endl;
+        } else if (_db->_idMapper.exists(fileName)) {
+            qDebug() << fileName << " already in database";
         } else {
             DB::ImageInfoPtr info = load( fileName, elm );
             _db->_images.append(info);
@@ -277,10 +272,10 @@ void XMLDB::FileReader::checkIfImagesAreSorted()
 #else
                                   i18n("<p>Your images/videos are not sorted, which means that navigating using the date bar "
                                        "will only work suboptimally.</p>"
-                                       "<p>You also don't have EXIF support available, which means that you can't read "
-                                       "image dates from JPEG metadata. It's strongly recommended to recompile KPhotoAlbum "
-                                       "with the <code>exiv2</code> library. After you have done so, you'll be asked what "
-                                       "to do to correct all missing information.</p>"),
+                                       "<p>You also do not have EXIF support available, which means that you cannot read "
+                                       "image dates from JPEG metadata. It is strongly recommended to recompile KPhotoAlbum "
+                                       "with the <code>exiv2</code> library. After you have done so, you will be asked what "
+                                       "to do to correct all the missing information.</p>"),
 #endif
                                   i18n("Images/Videos Are Not Sorted"),
                                   QString::fromLatin1( "checkWhetherImagesAreSorted" ) );
